// APAX_5013_8CH_RTD.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
///#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"


//////////////////////////////////////////////
//Take APAX-5013 as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 

#define NONE 0x00
#define DI	0x01
#define DO	0x02
#define AI	0x03
#define AO	0x04

//APAX 5013	range table
#define PT100_3851_NEG_200_TO_850_C		0x0200
#define PT100_3851_NEG_120_TO_130_C		0x0201
#define PT200_3851_NEG_200_TO_850_C		0x0220
#define PT200_3851_NEG_120_TO_130_C		0x0221
#define PT500_3851_NEG_200_TO_850_C		0x0240
#define PT500_3851_NEG_120_TO_130_C		0x0241
#define PT1000_3851_NEG_200_TO_850_C	0x0260
#define PT1000_3851_NEG_120_TO_130_C	0x0261
#define PT100_3916_NEG_200_TO_850_C		0x0280
#define PT100_3916_NEG_120_TO_130_C		0x0281
#define PT200_3916_NEG_200_TO_850_C		0x02A0
#define PT200_3916_NEG_120_TO_130_C		0x02A1
#define PT500_3916_NEG_200_TO_850_C		0x02C0
#define PT500_3916_NEG_120_TO_130_C		0x02C1
#define PT1000_3916_NEG_200_TO_850_C	0x02E0
#define PT1000_3916_NEG_120_TO_130_C	0x02E1
#define BALCON_NEG_30_TO_120_C			0x0300
#define NI_NEG_80_TO_100_C				0x0320
#define NI_ZERO_TO_100_C				0x0321

LONG g_lHandle = 0;
USHORT g_usSlotNum = 13;			//from 0 
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

void My_Show_SlotInfo(struct SlotInfo *i_pSlotInfo)
{
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	WORD 	wRangeType = 0;
	int 	iChannel = 0;
	BYTE 	byTotalChOfSlot = 0;
	BYTE 	*pbyChCounter = 0;
	BYTE 	byChCnt = 0;
	int 	iAddrShift = 0;
	BYTE 	byChannelType = 0;
 	
	dwHi = (i_pSlotInfo->dwModName & 0xFFFF0000) >> 16;
	dwLow = i_pSlotInfo->dwModName & 0x0000FFFF;

	//show module name
	if (0 == dwLow)
		printf("APAX %x module\n", (WORD)dwHi);
	else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
		printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
	else
		printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
	
	//get total channel number of this slot
	byTotalChOfSlot = i_pSlotInfo->byChTotal;

	for (iAddrShift=0; iAddrShift<10; iAddrShift+=2)
	{
		pbyChCounter = (BYTE *)&(i_pSlotInfo->byHwIoType_0) + iAddrShift + 1;
		byChCnt += *pbyChCounter;
		
		byChannelType = *(pbyChCounter - 1);
		switch(byChannelType) 
		{
			case DI:
				//When iAddrShift = 0, it try to extract byHwIoTotal_0 value of struct SlotInfo
				printf("This module has %d DI channels.\n", *pbyChCounter);
				break;
			
			case DO:
				printf("This module has %d DO channels.\n", *pbyChCounter);
				break;
			
			case AI:
				printf("This module has %d AI channels.\n", *pbyChCounter);
				break;

			case AO:
				printf("This module has %d AO channels.\n", *pbyChCounter);
			break;
		}

		if (byTotalChOfSlot <= byChCnt)
			break;
	}
	
	//show range of all channels  
	if ( (0x5013 == dwHi) ||							//APAX 5013
		 (0x5028 == dwHi) ||							//APAX 5028
		 (0x5017 == dwHi) ||							//APAX 5017
		 (0x5018 == dwHi) ||							//APAX 5018
		 ( (0x5017 == dwHi) && (0x0048 == dwLow) ) ) //APAX 5017H
	{
		for (iChannel=0; iChannel<byTotalChOfSlot; iChannel++)
		{
			wRangeType = *( (i_pSlotInfo->wChRange) + iChannel);
			
			//only implement APAX 5013
			switch(wRangeType)
			{
				case PT100_3851_NEG_200_TO_850_C:
				case PT100_3916_NEG_200_TO_850_C:	
					printf("Channel %d is PT-100 RTD, range between -200~850'C.\n", iChannel);
					break;

				case PT100_3851_NEG_120_TO_130_C:
				case PT100_3916_NEG_120_TO_130_C:	
					printf("Channel %d is PT-100 RTD, range between -120~130'C.\n", iChannel);
					break;

				case PT200_3851_NEG_200_TO_850_C:
				case PT200_3916_NEG_200_TO_850_C:	
					printf("Channel %d is PT-200 RTD, range between -200~850'C.\n", iChannel);
					break;

				case PT200_3851_NEG_120_TO_130_C:
				case PT200_3916_NEG_120_TO_130_C:	
					printf("Channel %d is PT-200 RTD, range between -120~130'C.\n", iChannel);
					break;

				case PT500_3851_NEG_200_TO_850_C:
				case PT500_3916_NEG_200_TO_850_C:
					printf("Channel %d is PT-500 RTD, range between -200~850'C.\n", iChannel);
					break;

				case PT500_3851_NEG_120_TO_130_C:
				case PT500_3916_NEG_120_TO_130_C:
					printf("Channel %d is PT-500 RTD, range between -120~130'C.\n", iChannel);
					break;

				case PT1000_3851_NEG_200_TO_850_C:
				case PT1000_3916_NEG_200_TO_850_C:	
					printf("Channel %d is PT-1000 RTD, range between -200~850'C.\n", iChannel);
					break;

				case PT1000_3851_NEG_120_TO_130_C:
				case PT1000_3916_NEG_120_TO_130_C:
					printf("Channel %d is PT-1000 RTD, range between -120~130'C.\n", iChannel);
					break;
									
				case BALCON_NEG_30_TO_120_C:
					printf("Channel %d range between -30~120'C.\n", iChannel);
					break;			
							
				case NI_NEG_80_TO_100_C:
					printf("Channel %d range between -80~100'C.\n", iChannel);
					break;			
							
				case NI_ZERO_TO_100_C:
					printf("Channel %d range between 0~100'C.\n", iChannel);
					break;				

				default:
					printf("Channel %d range is unknow.\n", iChannel);
					break;
			}
		}			
	}	
}

void Scale_RawData(struct SlotInfo *i_pSlotInfo, WORD i_Channel, WORD *i_pValue)
{
	WORD wRangeType = 0;
	double dRealValue = 0.0;


	wRangeType = *( (i_pSlotInfo->wChRange) + i_Channel);

	//only implement APAX 5013
	switch(wRangeType)
	{
		case PT100_3851_NEG_200_TO_850_C:
		case PT200_3851_NEG_200_TO_850_C:
		case PT500_3851_NEG_200_TO_850_C:
		case PT1000_3851_NEG_200_TO_850_C:
		case PT100_3916_NEG_200_TO_850_C:
		case PT200_3916_NEG_200_TO_850_C:
		case PT500_3916_NEG_200_TO_850_C:
		case PT1000_3916_NEG_200_TO_850_C:			
			dRealValue = -200.0 + (i_pValue[i_Channel] / 65535.0) * 1050.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case PT100_3851_NEG_120_TO_130_C:
		case PT200_3851_NEG_120_TO_130_C:
		case PT500_3851_NEG_120_TO_130_C:
		case PT1000_3851_NEG_120_TO_130_C:	
		case PT100_3916_NEG_120_TO_130_C:
		case PT200_3916_NEG_120_TO_130_C:
		case PT500_3916_NEG_120_TO_130_C:
		case PT1000_3916_NEG_120_TO_130_C:
			dRealValue = -120.0 + (i_pValue[i_Channel] / 65535.0) * 250.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case BALCON_NEG_30_TO_120_C:
			dRealValue = -30.0 + (i_pValue[i_Channel] / 65535.0) * 150.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NI_NEG_80_TO_100_C:
			dRealValue = -80.0 + (i_pValue[i_Channel] / 65535.0) * 180.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NI_ZERO_TO_100_C:
			dRealValue = (i_pValue[i_Channel] / 65535.0) * 100.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		default:
			printf("Channel %d range is unknow.\n", i_Channel);
			break;
	}
}

int main(int argc, char* argv[])
{
	const int iChNumber = 8;
	WORD wChannel = 0;
	WORD wValues[32] = {0};
	struct SlotInfo MySlotInfo ={0};
	WORD wNewRangeTab[8] = {0}; //APAX 5013 is a 8 channels RTD module
	BYTE byChannelStatus[32] = {0};
	int i = 0;
	DWORD dwMask = 0x0; 
	DWORD dwIntegration = 0;
	WORD wTimeout = 0;
	LONG lResult = 0;

	printf("Slot ID setting in this sample code is %d.\n", g_usSlotNum);
	
	if (ADV_SUCCESS == OpenLib() )
	{
		printf("\n*** Get module ID and current range setting ***\n");
		//get module ID and current range setting
		lResult = SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo);
		if (ERR_SUCCESS == lResult)
			My_Show_SlotInfo(&MySlotInfo);
		else
			printf("Get slot information error, erro code = %d\n", (WORD)lResult);
		
		usleep(100000);

		//Set inner timeout before config module
		printf("\n*** Set inner timeout ***\n");
		wTimeout = 1000; //ms
		lResult = SYS_SetInnerTimeout(g_lHandle, wTimeout);
		if (ERR_SUCCESS == lResult )
			printf("Set integration time succeed, timeout = %d ms.\n", wTimeout);
		else
			printf("Set integration time error, error code = %d.\n", (WORD)lResult);

		//Set integration time for frequency filter, dwIntegration = 0 is 60Hz, = 1 is 50Hz 
		printf("\n*** Set integration time ***\n");
		lResult = AI_SetIntegrationTime(g_lHandle, g_usSlotNum, dwIntegration);
		if (ERR_SUCCESS == lResult )
			printf("Set integration time succeed.\n");
		else
			printf("Set integration time error, error code = %d.\n", (WORD)lResult);

		usleep(100000);

		//Get channel status
		printf("\n*** Get channel status ***\n");
		lResult = AIO_GetChannelStatus(g_lHandle, g_usSlotNum, byChannelStatus);
		if (ERR_SUCCESS == lResult)
		{
			for (i=0; i<iChNumber; i++)
			{
				printf("Slot %d channel %d status is ", g_usSlotNum, i);

				switch(byChannelStatus[i])
				{
					case 0:
						printf("\"None\"\n");
						break;
					case 1:
						printf("\"Normal\"\n");
						break;
					case 2:
						printf("\"Over current\"\n");
						break;
					case 3:
						printf("\"Under current\"\n");
						break;
					case 4:
						printf("\"Burn out\"\n");
						break;
					case 5:
						printf("\"Open loop\"\n");
						break;
					case 6:
						printf("\"Not ready\"\n");
						break;
					default:
						printf("\"Unknown\"\n");
						break;
				}
			}
		}
		else
			printf("Get Slot %d channel status error, error code = %d.\n", g_usSlotNum, (WORD)lResult);

		usleep(100000);

		printf("\n*** Set new range table ***\n");
		wNewRangeTab[0] = PT100_3851_NEG_200_TO_850_C;
		wNewRangeTab[1] = PT100_3851_NEG_120_TO_130_C;
		wNewRangeTab[2] = PT500_3916_NEG_200_TO_850_C;
		wNewRangeTab[3] = PT200_3916_NEG_200_TO_850_C;
		wNewRangeTab[4] = PT1000_3851_NEG_120_TO_130_C;
		wNewRangeTab[5] = PT1000_3851_NEG_120_TO_130_C;
		wNewRangeTab[6] = BALCON_NEG_30_TO_120_C;
		wNewRangeTab[7] = NI_NEG_80_TO_100_C;

		//Set range follow APAX 5013 range table setting
		printf("Channel total = %d.\n", MySlotInfo.byChTotal);
		lResult = AIO_SetRanges(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal, wNewRangeTab);
		if (ERR_SUCCESS == lResult)
		{
			if (ERR_SUCCESS == SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo) )
			{
				My_Show_SlotInfo(&MySlotInfo);
			}
			else
				printf("Get slot information error.\n");
		}
		else
		{
			printf("Set new range table error, error code = %d.\n", (WORD)lResult);
		}
		
		usleep(100000);
		
		//Set channel mask
		printf("\n*** Set channel mask ***\n");
		dwMask = 0xFFFFFFFF; //enable all channel
		lResult = AI_SetChannelMask(g_lHandle, g_usSlotNum, dwMask);
		if (ERR_SUCCESS == lResult)
		{	
			for (i=0; i<iChNumber; i++)
			{
				if (dwMask & (0x01 << i) )
					printf("Channel %d = Enable.\n", i);
				else
					printf("Channel %d = Disable.\n", i);
			}
		}
		else
			printf("Set channel mask error, error code = %d\n", (WORD)lResult);
		
		usleep(100000);
		
		//Set inner timeout back to default
		printf("\n*** Set inner timeout ***\n");
		wTimeout = 50; //ms
		lResult = SYS_SetInnerTimeout(g_lHandle, wTimeout);
		if (ERR_SUCCESS == lResult )
			printf("Set integration time succeed, timeout = %d ms.\n", wTimeout);
		else
			printf("Set integration time error, error code = %d.\n", (WORD)lResult);

		usleep(100000);

		//Get value (single channel)
		printf("\nGet single channel value.\n");
		wValues[0] = 0x0;
		wChannel = 0;
		lResult = AIO_GetValue(g_lHandle, g_usSlotNum, wChannel, wValues + wChannel);
		if (ERR_SUCCESS == lResult)
			Scale_RawData(&MySlotInfo, wChannel, wValues);
		else
			printf("Channel %d get value error, error code = %d\n", wChannel, (WORD)lResult);

		usleep(100000);

		//Get value (multi channel)
		printf("\nGet multi channels value.\n");
		lResult = AIO_GetValues(g_lHandle, g_usSlotNum, wValues);
		if (ERR_SUCCESS == lResult)
		{
			for (wChannel=0; wChannel<MySlotInfo.byHwIoTotal_0; wChannel++)
				Scale_RawData(&MySlotInfo, wChannel, wValues);
		}
		else
			printf("Get multi channels value error, error code = %d\n", (WORD)lResult);
	}
	else
	{
		printf("Open library fail.\n");
	}
	
	CloseLib();
	printf("/***        END         ***/ \n\n");
	
    return 0;
}
