// APAX_5017H_12CH_AI.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
///#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"

#include <time.h>
#include <sys/time.h>

//////////////////////////////////////////////
//Take APAX-5017H as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 

#define NONE 0x00
#define DI	0x01
#define DO	0x02
#define AI	0x03
#define AO	0x04

//APAX 5017H	range table
#define ZERO_TO_500_MV	    0x0106 
#define NEG_10_TO_10_V		0x0143
#define _4_TO_20_MA			0x0180
#define ZERO_TO_20_MA		0x0182
#define ZERO_TO_10_V		0x0148

const int g_ChannelNum = 12; //the channel number of APAX-5017H is 12

LONG g_lHandle = 0;
USHORT g_usSlotNum = 11;  ///!!!4;			//slot id from 0 to 31


///!!!+++ for APAX-5017UH
#define APAX_HS_PKT_ENTRY_COUNT   4096 //4096 pkts for 5KHz data in 820 milli-second
#define APAX_HS_CHANNEL_COUNT     12

typedef struct _HSChannelValueRow
{
    unsigned int time_stamp_micro_sec;
    float dRealValue[APAX_HS_CHANNEL_COUNT]; 	
}	HS_CHANNEL_VALUE_ROW;

LONG g_rowCnt = APAX_HS_PKT_ENTRY_COUNT;
HS_CHANNEL_VALUE_ROW g_rowBuf[APAX_HS_PKT_ENTRY_COUNT]; 
///!!!---






//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

void Show_Module_id(struct SlotInfo *i_pSlotInfo)
{
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;

	dwHi = (i_pSlotInfo->dwModName & 0xFFFF0000) >> 16;
	dwLow = i_pSlotInfo->dwModName & 0x0000FFFF;

	//show module name
	if (0 == dwLow)
		printf("APAX %x module\n", (WORD)dwHi);
	else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
		printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
	else
		printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );

}

void Scale_RawData(struct SlotInfo *i_pSlotInfo, WORD i_Channel, WORD *i_pwValue)
{
	WORD wRangeType = 0;
	double dRealValue = 0.0;

	wRangeType = *( (i_pSlotInfo->wChRange) + i_Channel);

	//only for APAX-5017H
	switch(wRangeType)
	{
		case ZERO_TO_500_MV:
			dRealValue = (i_pwValue[i_Channel] / 4095.0) * 500.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mV.\n", i_Channel, i_pwValue[i_Channel], dRealValue);
	        break;

		case NEG_10_TO_10_V:
			dRealValue = -10.0 + (i_pwValue[i_Channel] / 4095.0) * 20.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pwValue[i_Channel], dRealValue);
	        break;

		case ZERO_TO_10_V:
			dRealValue = (i_pwValue[i_Channel] / 4095.0) * 10.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pwValue[i_Channel], dRealValue);
	        break;

		case _4_TO_20_MA:
			dRealValue = 4.0 + (i_pwValue[i_Channel] / 4095.0) * 16.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mA.\n", i_Channel, i_pwValue[i_Channel], dRealValue);
	        break;

		case ZERO_TO_20_MA:
			dRealValue = (i_pwValue[i_Channel] / 4095.0) * 20.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mA.\n", i_Channel, i_pwValue[i_Channel], dRealValue);
			break;

		default:
			printf("Channel %d range is unknow.\n", i_Channel);
			break;
	}
}

void Get_Channel_Range(LONG i_lDriverHandle,WORD i_wSlotID, struct SlotInfo *i_pSlotInfo)
{
	BYTE byTotalChOfSlot = 0;
	int iAddrShift = 0;
	int iChannel = 0;
	BYTE *pbyChCounter = 0;
	BYTE byChCnt = 0;
	WORD wRangeType = 0;
	BYTE byChannelType = 0;

	//get total channel number of this slot
    byTotalChOfSlot = i_pSlotInfo->byChTotal;

    for (iAddrShift=0; iAddrShift<10; iAddrShift+=2)
    {
        pbyChCounter = (BYTE *)&(i_pSlotInfo->byHwIoType_0) + iAddrShift + 1;
        byChCnt += *pbyChCounter;
        
        byChannelType = *(pbyChCounter - 1);
        switch(byChannelType) 
        {
            case DI:
                //When iAddrShift = 0, it try to extract byHwIoTotal_0 value of SlotInfo struct 
                printf("This module has %d DI channels.\n", *pbyChCounter);
                break;
            
            case DO:
                printf("This module has %d DO channels.\n", *pbyChCounter);
                break;
            
            case AI:
                printf("This module has %d AI channels.\n", *pbyChCounter);
                break;

            case AO:
                printf("This module has %d AO channels.\n", *pbyChCounter);
                break;
        }

        if (byTotalChOfSlot <= byChCnt)
            break;
    }
    
    for (iChannel=0; iChannel<byTotalChOfSlot; iChannel++)
    {
        wRangeType = *( (i_pSlotInfo->wChRange) + iChannel);
        
        //only implement APAX 5017H
        switch(wRangeType)
        {
            case  ZERO_TO_500_MV: 
                printf("The range of channel %d is 0 ~ 500 mV.\n", iChannel);
                break;

            case NEG_10_TO_10_V:
                printf("The range of channel %d is +/- 10 V.\n", iChannel);
                break;

            case _4_TO_20_MA:
                printf("The range of channel %d is 4 ~ 20 mA.\n", iChannel);
                break;

            case ZERO_TO_20_MA:
                printf("The range of channel %d is 0 ~ 20 mA.\n", iChannel);
                break;

            case ZERO_TO_10_V:
                printf("The range of channel %d is 0 ~ 10 V.\n", iChannel);
                break;

            default:
                printf("Channel %d range is unknow.\n", iChannel);
                break;
        }
    }
}

void Get_Channel_Status(LONG i_lDriverHandle,WORD i_wSlotID)
{
	LONG lResult = 0;
    BYTE byChannelStatus[32] = {0};
    int i;

    printf("\n*** Get channel status ***\n");
    lResult = AIO_GetChannelStatus(g_lHandle, g_usSlotNum, byChannelStatus);
    if (ERR_SUCCESS == lResult)
    {
        for (i=0; i<g_ChannelNum; i++)
        {
            printf("Slot %d channel %d status is ", g_usSlotNum, i);

            switch(byChannelStatus[i])
            {
                case 0:
                    printf("\"None\"\n");
                    break;
                case 1:
                    printf("\"Normal\"\n");
                    break;
                case 2:
                    printf("\"Over current\"\n");
                    break;
                case 3:
                    printf("\"Under current\"\n");
                    break;
                case 4:
                    printf("\"Burn out\"\n");
                    break;
                case 5:
                    printf("\"Open loop\"\n");
                    break;
                case 6:
                    printf("\"Not ready\"\n");
                    break;
                default:
                    printf("\"Unknown\"\n");
                    break;
            }
        }
    }
    else
        printf("Get Slot %d channel status error, error code = %d\n", g_usSlotNum, (WORD)lResult);

    usleep(100000);
}

void SetBurnoutMode(LONG i_lDriverHandle,WORD i_wSlotID)
{
	//==============================
	// user can set parameters below
	//==============================
	//set dwBurnoutMode = 0      for "down scale"
	//set dwBurnoutMode = 0xFFFF for "up scale"

	DWORD dwBurnoutMode = 0xFFFF; //up scale

	LONG lSetBurnoutModeResult = AI_SetBurnoutValue(i_lDriverHandle, i_wSlotID, dwBurnoutMode);
	if (ERR_SUCCESS == lSetBurnoutModeResult)	
	{	
		printf("Succeed to set burnout detect mode.\n");
		usleep(100000);
	}
	else
		printf("Fail to set burnout detect mode, error code = %d\n", lSetBurnoutModeResult);
}

void GetBurnoutMode(LONG i_lDriverHandle,WORD i_wSlotID)
{
	DWORD dwBurnoutMode = 0; 

	LONG lGetBurnoutModeResult = AI_GetBurnoutValue(i_lDriverHandle, i_wSlotID, &dwBurnoutMode);
	if (ERR_SUCCESS == lGetBurnoutModeResult)	
	{	
		if (dwBurnoutMode > 0)
		{
			printf("The burnout detect mode is \" %s \".\n","Up Scale");
		}
		else
			printf("The burnout detect mode is \" %s \".\n","Down Scale");
		
		usleep(100000);
	}
	else
		printf("Fail to get burnout detect mode, error code = %d\n", lGetBurnoutModeResult);
}

void GetSamplingRate(LONG i_lDriverHandle,WORD i_wSlotID)
{
	DWORD dwSampleRate = 0;

	LONG lGetSamplingRateResult = AI_GetSampleRate(i_lDriverHandle, i_wSlotID, &dwSampleRate);
	if (ERR_SUCCESS == lGetSamplingRateResult)	
	{	
		printf("The sampling rate is %d (Hz/Ch).\n",dwSampleRate);
		usleep(100000);
	}
	else
		printf("Fail to get sampling rate, error code = %d\n", lGetSamplingRateResult);
}

void SetSamplingRate(LONG i_lDriverHandle,WORD i_wSlotID)
{
	//==============================
	// user can set parameter below
	//==============================
	DWORD dwSampleRate = 1000; //in (Hz/Ch)

	LONG lSetSamplingRateResult = AI_SetSampleRate(i_lDriverHandle, i_wSlotID, dwSampleRate);
	if (ERR_SUCCESS == lSetSamplingRateResult)	
	{	
		printf("Succeed to set sampling rate.\n");
		usleep(100000);
	}
	else
		printf("Fail to set sampling rate, error code = %d\n", lSetSamplingRateResult);
}

void SetChannelMask(LONG i_lDriverHandle,WORD i_wSlotID)
{
    DWORD dwMask = 0xFFF; //enable all channel
    LONG lResult = 0;
    struct SlotInfo MySlotInfo ={0};
    int i;
    
    lResult = AI_SetChannelMask(i_lDriverHandle, i_wSlotID, dwMask);
		if (ERR_SUCCESS == lResult)
		{	
			for (i=0; i<12; i++)
			{
				if (dwMask & (0x01 << i) )
					printf("Channel %d = Enable.\n", i);
				else
					printf("Channel %d = Disable.\n", i);
			}
		}
		else
			printf("Set Set channel mask error, error code = %d.\n", (WORD)lResult);

		usleep(100000);

		//Get Channel Mask
		printf("\nGet channel mask.\n");
		lResult = SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo);
		if (ERR_SUCCESS == lResult)
		{
			DWORD dwMask = 0;
			dwMask = MySlotInfo.dwChMask;
			for(i=0; i < 12; i++)
			{
				if (dwMask & ( 0x01 << i))
					printf("Channel %d = Enable.\n", i);
				else
					printf("Channel %d = Disable.\n", i);
			}
		}
		else
			printf("Get slot information error, error code = %d\n", (WORD)lResult);
		usleep(100000);    
}

int main(int argc, char* argv[])
{
    WORD wChannel = 0;
	  WORD wValues[32] = {0};
	  struct SlotInfo MySlotInfo ={0};
	  LONG lResult = 0;

	  printf("Slot ID setting in this sample code is %d.\n", g_usSlotNum);
	
	  if (ADV_SUCCESS == OpenLib() )
	  {
		    //===========================================================
		    //get module ID and current range setting
		    printf("\n*** Get module ID and current range setting ***\n");
		    
		    lResult = SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo);
		    if (ERR_SUCCESS == lResult)
		    {
			      Show_Module_id(&MySlotInfo);
		    }
		    else
			      printf("Get slot information error, error code = %d\n", (WORD)lResult);

		    Get_Channel_Range(g_lHandle, g_usSlotNum, &MySlotInfo);
		
		    usleep(100000);
		
		
		    //Set inner timeout before config module
		    /*
		    WORD wTimeout = 0;
		    
		    printf("\n*** Set inner timeout ***\n");
		    wTimeout = 1000; //ms
		    lResult = SYS_SetInnerTimeout(g_lHandle, wTimeout);
		    if (ERR_SUCCESS == lResult )
			      printf("Set inner timeout succeed, timeout = %d ms.\n", wTimeout);
		    else
			      printf("Set inner timeout error, error code = %d.\n", (WORD)lResult);
		
		    usleep(100000);
		    */
		

        //===========================================================
		    //Get dynamic value (single channel)
		    printf("\n>>>> Test to get single dynamic channel value.\n\n");
		    wValues[0] = 0x0;
		    wChannel = 10; //3;  //0;
		    lResult = AIO_GetValue(g_lHandle, g_usSlotNum, wChannel, wValues + wChannel);
		    if (ERR_SUCCESS == lResult)
			      Scale_RawData(&MySlotInfo, wChannel, wValues);
		    else
			      printf("Channel %d get value error, error code = %d\n", wChannel, (WORD)lResult);



		    struct timeval start;
		    struct timeval end;
        int i =0;

		    gettimeofday(&start, NULL);
		    //printf("start_time : %ld\n", start.tv_sec);

		
		    //===========================================================
		    // Get AI Freq
		    /*
		    int cnt = 0;
		    
		    for(i=0;i<100000;i++)
		    {
			      lResult = AIO_GetValue(g_lHandle, g_usSlotNum, wChannel, wValues + wChannel);
			      double _dRealValue = -10.0 + (wValues[wChannel] / 4095.0) * 20.0;
			      //printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", wChannel, wValues[wChannel], 	_dRealValue);
			
			      usleep(10);
			
			      if (flg && _dRealValue <= 2.0)
			      { 
			          cnt++; 
			          flg = FALSE;
			      }
			      else if(_dRealValue > 2.0) 
			          flg = TRUE;
		    }
		
		    double freq = cnt / total_time;
		    printf("cnt : %d\n", cnt);
		    printf("freq : %lf\n", freq);
		    */
		
		
		    //===========================================================
		    // Log all 12 channels data continuously at specific interval into file, apax5017uh.txt
		    
		    printf("\n>>>> To log all 12 channels data continuously at specific interval into file, apax5017uh.txt \n\n");
		    
		    FILE * fp = fopen("apax5017uh.txt","w");
		    size_t len, file_len=0;
		    int prevTime, capture_time_ms, capture_file_len, capture_row_cnt, left_row_cnt;
		    char tmpStrBuf[512];
		
		    if( fp==NULL )
		    {
		        printf("fail to open file!\n");
		        return -1;	
		    }		
		
		    prevTime = 0;
		    capture_time_ms = 5000;      // total sleep time
		    capture_file_len = 10000000; // 10 MB
		    left_row_cnt = capture_row_cnt = 20000;   // count of rows of data
		    while( capture_time_ms>0 && capture_file_len>0 && left_row_cnt>0 )
		    {
		        lResult = HSAI_GetBulkChannelRows(g_lHandle, g_usSlotNum, (unsigned char *)g_rowBuf, g_rowCnt);

		        if( lResult > 0 )
		        {
		    	      if( left_row_cnt < lResult )   
		    	          lResult = left_row_cnt;	
		    	  
		            for(i=0; i<lResult; i++)
		            {
		                sprintf(tmpStrBuf, "%8u,%8u,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f,%6.3f\n\0",g_rowBuf[i].time_stamp_micro_sec-prevTime, 
		                        g_rowBuf[i].time_stamp_micro_sec,
		                        g_rowBuf[i].dRealValue[0],g_rowBuf[i].dRealValue[1],g_rowBuf[i].dRealValue[2],g_rowBuf[i].dRealValue[3],
		                        g_rowBuf[i].dRealValue[4],g_rowBuf[i].dRealValue[5],g_rowBuf[i].dRealValue[6],g_rowBuf[i].dRealValue[7],
		                        g_rowBuf[i].dRealValue[8],g_rowBuf[i].dRealValue[9],g_rowBuf[i].dRealValue[10],g_rowBuf[i].dRealValue[11] );	
		             
		                len = strlen(tmpStrBuf);        
		                  
		                //printf("i=%d: len=%d, '%s'", i, len, tmpStrBuf );
		                    
		                prevTime = g_rowBuf[i].time_stamp_micro_sec;       
		            
		                fwrite(tmpStrBuf, 1, len, fp);
		                file_len += len;
                    capture_file_len -= len; 	
                    if( capture_file_len <= 0 )
                        break;		            
		            }	
		            left_row_cnt -= lResult;
		        } 
		    
			      usleep(1000);  // sleep 1 ms
			      capture_time_ms -= 1;
		    }	 
		    fclose(fp);
		    printf("::   Total captured rows of channel data = %d;  File length = %d  \n", capture_row_cnt, file_len );
		    //-------------------------------------------------------------------------------
		

		    gettimeofday(&end, NULL);
		    //printf("end_time : %ld\n", end.tv_sec);

		    printf("::   total_time : %d sec\n", end.tv_sec - start.tv_sec);
	  }
	  else
	  {
		    printf("Open library fail.\n");
	  }
	
	  CloseLib();
	  printf("\n\n/***        END         ***/ \n\n");
	
	  return 0;
}
