// APAX_5018_12CH_Thermocouple.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
///#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"


//////////////////////////////////////////////
//Take APAX-5018 as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 

#define NONE 0x00
#define DI	0x01
#define DO	0x02
#define AI	0x03
#define AO	0x04

//APAX 5018	range table
#define NEG_50_TO_50_MV		0x0101
#define NEG_100_TO_100_MV	0x0102
#define NEG_500_TO_500_MV	0x0104
#define NEG_1_TO_1_V		0x0140
#define NEG_2_5_TO_2_5_V	0x0141
#define _4_TO_20_MA			0x0180
#define NEG_20_TO_20_MA		0x0181
#define ZERO_TO_20_MA		0x0182
#define TYPE_J_NEG_210_TO_1200_C	0x0401
#define TYPE_K_NEG_270_TO_1372_C	0x0421
#define TYPE_T_NEG_270_TO_400_C		0x0441
#define TYPE_E_NEG_270_TO_1000_C	0x0461
#define TYPE_R_ZERO_TO_1768_C		0x0481
#define TYPE_S_ZERO_TO_1768_C		0x04A1
#define TYPE_B_300_TO_1820_C		0x04C1


LONG g_lHandle = 0;
USHORT g_usSlotNum = 14;			//from 0 
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

void My_Show_SlotInfo(struct SlotInfo *i_pSlotInfo)
{
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	WORD wRangeType = 0;
	int iChannel = 0;
	BYTE byTotalChOfSlot = 0;
	BYTE *pbyChCounter = 0;
	BYTE byChCnt = 0;
	int iAddrShift = 0;
	BYTE byChannelType = 0;

	dwHi = (i_pSlotInfo->dwModName & 0xFFFF0000) >> 16;
	dwLow = i_pSlotInfo->dwModName & 0x0000FFFF;

	//show module name
	if (0 == dwLow)
		printf("APAX %x module\n", (WORD)dwHi);
	else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
		printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
	else
		printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
	
	//get total channel number of this slot
	byTotalChOfSlot = i_pSlotInfo->byChTotal;

	for (iAddrShift=0; iAddrShift<10; iAddrShift+=2)
	{
		pbyChCounter = (BYTE *)&(i_pSlotInfo->byHwIoType_0) + iAddrShift + 1;
		byChCnt += *pbyChCounter;
		
		byChannelType = *(pbyChCounter - 1);
		switch(byChannelType) 
		{
			case DI:
				//When iAddrShift = 0, it try to extract byHwIoTotal_0 value of struct SlotInfo
				printf("This module has %d DI channels.\n", *pbyChCounter);
				break;
			
			case DO:
				printf("This module has %d DO channels.\n", *pbyChCounter);
				break;
			
			case AI:
				printf("This module has %d AI channels.\n", *pbyChCounter);
				break;

			case AO:
				printf("This module has %d AO channels.\n", *pbyChCounter);
				break;
		}

		if (byTotalChOfSlot <= byChCnt)
			break;
	}
	
	//show range of all channels  
	if ( (0x5013 == dwHi) ||							//APAX 5013
		 (0x5028 == dwHi) ||							//APAX 5028
		 (0x5017 == dwHi) ||							//APAX 5017
		 (0x5018 == dwHi) ||							//APAX 5018
		 ( (0x5017 == dwHi) && (0x0048 == dwLow) ) ) //APAX 5017H
	{
		for (iChannel=0; iChannel<byTotalChOfSlot; iChannel++)
		{
			wRangeType = *( (i_pSlotInfo->wChRange) + iChannel);
			
			//only implement APAX 5018
			switch(wRangeType)
			{
				case NEG_50_TO_50_MV:
					printf("Channel %d range is +/- 50 mV.\n", iChannel);
					break;

				case NEG_100_TO_100_MV:
					printf("Channel %d range is +/- 100 mV.\n", iChannel);
					break;

				case NEG_500_TO_500_MV:
					printf("Channel %d range is +/- 500 mV.\n", iChannel);
					break;

				case NEG_1_TO_1_V:
					printf("Channel %d range is +/- 1 V.\n", iChannel);
					break;

				case NEG_2_5_TO_2_5_V:
					printf("Channel %d range is +/- 2.5 V.\n", iChannel);
					break;

				case _4_TO_20_MA:
					printf("Channel %d range is 4~20 mA.\n", iChannel);
					break;

				case NEG_20_TO_20_MA:
					printf("Channel %d range is +/- 20 mA.\n", iChannel);
					break;

				case ZERO_TO_20_MA:
					printf("Channel %d range is 0~20 mA.\n", iChannel);
					break;
				
				case TYPE_J_NEG_210_TO_1200_C:
					printf("Channel %d is type J thermocouple, range between -210~1200'C .\n", iChannel);
					break;

				case TYPE_K_NEG_270_TO_1372_C:
					printf("Channel %d is type K thermocouple, range between -270~1372'C .\n", iChannel);
					break;	

				case TYPE_T_NEG_270_TO_400_C:
					printf("Channel %d is type T thermocouple, range between -270~400'C .\n", iChannel);
					break;

				case TYPE_E_NEG_270_TO_1000_C:
					printf("Channel %d is type E thermocouple, range between -270~1000'C .\n", iChannel);
					break;
				
				case TYPE_R_ZERO_TO_1768_C:
					printf("Channel %d is type R thermocouple, range between 0~1768'C .\n", iChannel);
					break;	
				
				case TYPE_S_ZERO_TO_1768_C:
					printf("Channel %d is type S thermocouple, range between 0~1768'C .\n", iChannel);
					break;
				
				case TYPE_B_300_TO_1820_C:
					printf("Channel %d is type B thermocouple, range between 300~1820'C .\n", iChannel);
					break;	

				default:
					printf("Channel %d range is unknow.\n", iChannel);
					break;
			}
		}			
	}	
}

void Scale_RawData(struct SlotInfo *i_pSlotInfo, WORD i_Channel, WORD *i_pValue)
{
	WORD wRangeType = 0;
	double dRealValue = 0.0;


	wRangeType = *( (i_pSlotInfo->wChRange) + i_Channel);

	//only implement APAX 5018
	switch(wRangeType)
	{
		case NEG_50_TO_50_MV:
			dRealValue = -50.0 + (i_pValue[i_Channel] / 65535.0) * 100.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mV.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NEG_100_TO_100_MV:
			dRealValue = -100.0 + (i_pValue[i_Channel] / 65535.0) * 200.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mV.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NEG_500_TO_500_MV:
			dRealValue = -500.0 + (i_pValue[i_Channel] / 65535.0) * 1000.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mV.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NEG_1_TO_1_V:
			dRealValue = -1.0 + (i_pValue[i_Channel] / 65535.0) * 2.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NEG_2_5_TO_2_5_V:
			dRealValue = -2.5 + (i_pValue[i_Channel] / 65535.0) * 5.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case _4_TO_20_MA:
			dRealValue = 4.0 + (i_pValue[i_Channel] / 65535.0) * 16.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mA.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NEG_20_TO_20_MA:
			dRealValue = -20.0 + (i_pValue[i_Channel] / 65535.0) * 40.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mA.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case ZERO_TO_20_MA:
			dRealValue = (i_pValue[i_Channel] / 65535.0) * 20.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f mA.\n", i_Channel, i_pValue[i_Channel], dRealValue);
			break;

		case TYPE_J_NEG_210_TO_1200_C:
			dRealValue = -210.0 + (i_pValue[i_Channel] / 65535.0) * 1410.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
			break;

		case TYPE_K_NEG_270_TO_1372_C:
			dRealValue = -270.0 + (i_pValue[i_Channel] / 65535.0) * 1642.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
			break;
				
		case TYPE_T_NEG_270_TO_400_C:
			dRealValue = -270.0 + (i_pValue[i_Channel] / 65535.0) * 670.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
			break;
		
		case TYPE_E_NEG_270_TO_1000_C:
			dRealValue = -270.0 + (i_pValue[i_Channel] / 65535.0) * 1270.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
			break;
		
		case TYPE_R_ZERO_TO_1768_C:
		case TYPE_S_ZERO_TO_1768_C:	
			dRealValue = (i_pValue[i_Channel] / 65535.0) * 1768.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
			break;
		
		case TYPE_B_300_TO_1820_C:
			dRealValue = 300 + (i_pValue[i_Channel] / 65535.0) * 1520.0;
			printf("Channel %d raw data is 0x%x, scaled value is %.4f'C.\n", i_Channel, i_pValue[i_Channel], dRealValue);
			break;

		default:
			printf("Channel %d range is unknow.\n", i_Channel);
			break;
	}
}

int main(int argc, char* argv[])
{
	WORD wChannel = 0;
	WORD wTimeout = 0;
	WORD wValues[32] = {0};
	struct SlotInfo MySlotInfo ={0};
	WORD wNewRangeTab[12] = {0}; //APAX 5018 is a 12 channels AI module
	BYTE byChannelStatus[32] = {0};
	int i = 0;
	DWORD dwMask = 0x0; 
	DWORD dwIntegration = 0;
	LONG lResult = 0;

	printf("Slot ID setting in this sample code is %d.\n", g_usSlotNum);
	
	if (ADV_SUCCESS == OpenLib() )
	{
		printf("\n*** Get module ID and current range setting ***\n");
		//get module ID and current range setting
		lResult = SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo);
		if (ERR_SUCCESS == lResult )
		{
			My_Show_SlotInfo(&MySlotInfo);
		}
		else
			printf("Get slot information error, error code = %d\n", (WORD)lResult);
		
		usleep(100000);
		
		//Set inner timeout before config module
		printf("\n*** Set inner timeout ***\n");
		wTimeout = 1000; //ms
		lResult = SYS_SetInnerTimeout(g_lHandle, wTimeout);
		if (ERR_SUCCESS == lResult )
			printf("Set integration time succeed, timeout = %d ms.\n", wTimeout);
		else
			printf("Set integration time error, error code = %d.\n", (WORD)lResult);
		
		usleep(100000);

		//Set integration time for frequency filter, dwIntegration = 0 is 60Hz, = 1 is 50Hz 
		printf("\n*** Set integration time ***\n");
		lResult = AI_SetIntegrationTime(g_lHandle, g_usSlotNum, dwIntegration);
		if (ERR_SUCCESS == lResult)
			printf("Set integration time succeed.\n");
		else
			printf("Set integration time error, error code = %d\n", (WORD)lResult);

		usleep(100000);

		//Get channel status
		printf("\n*** Get channel status ***\n");
		lResult = AIO_GetChannelStatus(g_lHandle, g_usSlotNum, byChannelStatus);
		if (ERR_SUCCESS == lResult)
		{
			for (i=0; i<12; i++)
			{
				printf("Slot %d channel %d status is ", g_usSlotNum, i);

				switch(byChannelStatus[i])
				{
					case 0:
						printf("\"None\"\n");
						break;
					case 1:
						printf("\"Normal\"\n");
						break;
					case 2:
						printf("\"Over current\"\n");
						break;
					case 3:
						printf("\"Under current\"\n");
						break;
					case 4:
						printf("\"Burn out\"\n");
						break;
					case 5:
						printf("\"Open loop\"\n");
						break;
					case 6:
						printf("\"Not ready\"\n");
						break;
					default:
						printf("\"Unknown\"\n");
						break;
				}
			}
		}
		else
			printf("Get Slot %d channel status error, error code = %d\n", g_usSlotNum, (WORD)lResult);

		usleep(100000);

		printf("\n*** Set new range table ***\n");
		wNewRangeTab[0] = ZERO_TO_20_MA;
		wNewRangeTab[1] = NEG_50_TO_50_MV;
		wNewRangeTab[2] = _4_TO_20_MA;
		wNewRangeTab[3] = NEG_1_TO_1_V;
		wNewRangeTab[4] = NEG_500_TO_500_MV;
		wNewRangeTab[5] = ZERO_TO_20_MA;
		wNewRangeTab[6] = NEG_2_5_TO_2_5_V;
		wNewRangeTab[7] = TYPE_T_NEG_270_TO_400_C;
		wNewRangeTab[8] = TYPE_K_NEG_270_TO_1372_C;
		wNewRangeTab[9] = NEG_1_TO_1_V;
		wNewRangeTab[10] = _4_TO_20_MA;
		wNewRangeTab[11] = NEG_50_TO_50_MV;


		//Set range follow APAX 5018 range table setting
		printf("Channel total = %d.\n", MySlotInfo.byChTotal);
		
		lResult = AIO_SetRanges(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal, wNewRangeTab); 
		if (ERR_SUCCESS == lResult )
		{
			usleep(100000);
			if (ERR_SUCCESS == SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo) )
			{
				My_Show_SlotInfo(&MySlotInfo);
			}
			else
				printf("Get slot information error, erro code = %d\n", (WORD)lResult);
		}
		else
			printf("Set new range table error, error code = %x.\n", (WORD)lResult);

		usleep(100000);
		
		//Set channel mask
		printf("\nSet channel mask.\n");
		dwMask = 0xFFFFFFFF; //enable all channel
		lResult = AI_SetChannelMask(g_lHandle, g_usSlotNum, dwMask);
		if (ERR_SUCCESS == lResult)
		{	
			for (i=0; i<12; i++)
			{
				if (dwMask & (0x01 << i) )
					printf("Channel %d = Enable.\n", i);
				else
					printf("Channel %d = Disable.\n", i);
			}
		}
		else
			printf("Set channel mask error, error code = %d\n", (WORD)lResult);

		usleep(100000);
		
		//Set inner timeout back to default
		printf("\n*** Set inner timeout ***\n");
		wTimeout = 50; //ms
		lResult = SYS_SetInnerTimeout(g_lHandle, wTimeout);
		if (ERR_SUCCESS == lResult )
			printf("Set integration time succeed, timeout = %d ms.\n", wTimeout);
		else
			printf("Set integration time error, error code = %d.\n", (WORD)lResult);
			
		usleep(100000);

		//Get value (single channel)
		printf("\nGet single channel value.\n");
		wChannel = 7;
		lResult = AIO_GetValue(g_lHandle, g_usSlotNum, wChannel, wValues + wChannel);
		if (ERR_SUCCESS == lResult)
			Scale_RawData(&MySlotInfo, wChannel, wValues);
		else
			printf("Channel %d get value error, error code = %d\n", wChannel, (WORD)lResult);

		usleep(100000);

		//Get value (multi channel)
		printf("\nGet multi channels value.\n");
		lResult = AIO_GetValues(g_lHandle, g_usSlotNum, wValues);
		if (ERR_SUCCESS == lResult)
		{
			for (wChannel=0; wChannel<MySlotInfo.byHwIoTotal_0; wChannel++)
			{
				Scale_RawData(&MySlotInfo, wChannel, wValues);
			}
		}
		else
			printf("Get multi channels value error, error code = %d\n", (WORD)lResult);
	}
	else
	{
		printf("Open library fail.\n");
	}
	
	CloseLib();
	printf("/***        END         ***/ \n\n");
	
	return 0;
}
