// APAX_5028_8CH_AO.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
///#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"

//////////////////////////////////////////////
//Take APAX-5028 as example 
//////////////////////////////////////////////

#define ADV_SUCCESS 0
#define ADV_FAIL	1 

#define NONE 0x00
#define DI	0x01
#define DO	0x02
#define AI	0x03
#define AO	0x04

///#define ENABLE_CALIBRATION_PROCESS

//APAX 5028	range table
#define NEG_2P5_TO_2P5_V	0x0141
#define NEG_5_TO_5_V		0x0142
#define NEG_10_TO_10_V		0x0143
#define ZERO_TO_2P5_V		0x0146
#define ZERO_TO_5_V			0x0147
#define ZERO_TO_10_V		0x0148
#define _4_TO_20_MA			0x0180
#define ZERO_TO_20_MA		0x0182


LONG g_lHandle = 0;
USHORT g_usSlotNum = 1;	 ///!!!+++ Yaojen@ Dec 2016		//slot id from 0 to 31
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

void My_Show_SlotInfo(struct SlotInfo *i_pSlotInfo)
{
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	WORD 	wRangeType = 0;
	int 	iChannel = 0;
	BYTE 	byTotalChOfSlot = 0;
	BYTE 	*pbyChCounter = 0;
	BYTE 	byChCnt = 0;
	int 	iAddrShift = 0;
	BYTE 	byChannelType = 0;

	dwHi = (i_pSlotInfo->dwModName & 0xFFFF0000) >> 16;
	dwLow = i_pSlotInfo->dwModName & 0x0000FFFF;

	//show module name
	if (0 == dwLow)
		printf("APAX %x module\n", (WORD)dwHi);
	else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
		printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
	else
		printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
	
	//get total channel number of this slot
	byTotalChOfSlot = i_pSlotInfo->byChTotal;

	for (iAddrShift=0; iAddrShift<10; iAddrShift+=2)
	{
		pbyChCounter = (BYTE *)&(i_pSlotInfo->byHwIoType_0) + iAddrShift + 1;
		byChCnt += *pbyChCounter;
		
		byChannelType = *(pbyChCounter - 1);
		switch(byChannelType) 
		{
			case DI:
				//When iAddrShift = 0, it try to extract byHwIoTotal_0 value of struct SlotInfo
				printf("This module has %d DI channels.\n", *pbyChCounter);
				break;
			
			case DO:
				printf("This module has %d DO channels.\n", *pbyChCounter);
				break;
			
			case AI:
				printf("This module has %d AI channels.\n", *pbyChCounter);
				break;

			case AO:
				printf("This module has %d AO channels.\n", *pbyChCounter);
			break;
		}

		if (byTotalChOfSlot <= byChCnt)
			break;
	}
	
	//show range of all channels  
	if ( (0x5013 == dwHi) ||							//APAX 5013
		 (0x5028 == dwHi) ||							//APAX 5028
		 (0x5017 == dwHi) ||							//APAX 5017
		 (0x5018 == dwHi) ||							//APAX 5018
		 ( (0x5017 == dwHi) && (0x0048 == dwLow) ) ) //APAX 5017H
	{
		for (iChannel=0; iChannel<byTotalChOfSlot; iChannel++)
		{
			wRangeType = *( (i_pSlotInfo->wChRange) + iChannel);
			
			//only implement APAX 5028
			if (NEG_2P5_TO_2P5_V == wRangeType)
				printf("Channel %d range is +/- 2.5 V.\n", iChannel);
			else if (NEG_5_TO_5_V == wRangeType)
				printf("Channel %d range is +/- 5 V.\n", iChannel);
			else if (NEG_10_TO_10_V == wRangeType)
				printf("Channel %d range is +/- 10 V.\n", iChannel);
			else if (ZERO_TO_2P5_V == wRangeType)
				printf("Channel %d range is 0~2.5 V.\n", iChannel);
			else if (ZERO_TO_5_V == wRangeType)
				printf("Channel %d range is 0~5 V.\n", iChannel);
			else if (ZERO_TO_10_V == wRangeType)
				printf("Channel %d range is 0~10 V.\n", iChannel);
			else if (_4_TO_20_MA == wRangeType)
				printf("Channel %d range is -4~20 mA.\n", iChannel);  ///!!!+++ Yaojen@: 4~20
			else if (ZERO_TO_20_MA == wRangeType)
				printf("Channel %d range is 0~20 mA.\n", iChannel);
			else
				printf("Channel %d range is unknow.\n", iChannel);
		}	
				
	}
	
}

void Scale_RawData(struct SlotInfo *i_pSlotInfo, WORD i_Channel, WORD *i_pValue)
{
	WORD 	wRangeType = 0;
	double 	dRealValue = 0.0;


	wRangeType = *( (i_pSlotInfo->wChRange) + i_Channel);

	//only implement APAX 5028
	switch(wRangeType)
	{
		case NEG_2P5_TO_2P5_V:
		dRealValue = -2.5 + (i_pValue[i_Channel] / 65535.0) * 5.0;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NEG_5_TO_5_V:
		dRealValue = -5 + (i_pValue[i_Channel] / 65535.0) * 10.0;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case NEG_10_TO_10_V:
		dRealValue = -10 + (i_pValue[i_Channel] / 65535.0) * 20.0;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case ZERO_TO_2P5_V:
		dRealValue = (i_pValue[i_Channel] / 65535.0) * 2.5;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case ZERO_TO_5_V:
		dRealValue = (i_pValue[i_Channel] / 65535.0) * 5.0;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case ZERO_TO_10_V:
		dRealValue = (i_pValue[i_Channel] / 65535.0) * 10.0;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f V.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case _4_TO_20_MA:
		///!!!+++ Yaojen@ Dec 2016
		///dRealValue = 4 + (i_pValue[i_Channel] / 65535.0) * 16.0;
		dRealValue = -4 + (i_pValue[i_Channel] / 65535.0) * 16.0;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f mA.\n", i_Channel, i_pValue[i_Channel], dRealValue);
	        break;

		case ZERO_TO_20_MA:
		dRealValue = (i_pValue[i_Channel] / 65535.0) * 20.0;
		printf("Channel %d raw data is 0x%x, scaled value is %.4f mA.\n", i_Channel, i_pValue[i_Channel], dRealValue);
		break;

		default:
		printf("Channel %d range is unknow.\n", i_Channel);
		break;
	}
}

int main(int argc, char* argv[])
{
	WORD	wChannel = 0;
	DWORD	dwModuleID = 0;
	DWORD	dwHi = 0;
	DWORD	dwLow = 0;
	struct SlotInfo MySlotInfo ={0};
	WORD	wStartupVal[32] = {0};
	WORD 	wNewRangeTab[8] = {0}; //APAX 5028 is a 8 channels AO module
	DWORD 	dwChannelMask = 0x000000AA; ///!!!+++ Yaojen@ Dec 2016: 0x000000FF;
	DWORD 	dwSlotMask = (0x01 << g_usSlotNum);
	WORD 	wOutput[32] = {0};
	WORD 	wValues[32] = {0};
	BYTE 	byChannelStatus[32] = {0};
	WORD 	wTimeout = 50; //default value
	int i = 0;
	LONG lResult = 0;

	if (ADV_SUCCESS != OpenLib() )
	{
		printf("Open library fail.\n");
		return 0;		
	}

	printf("Slot ID setting in this sample code is %d.\n", g_usSlotNum);
	
	//get module ID
	lResult = SYS_GetModuleID(g_lHandle, g_usSlotNum, &dwModuleID);
	if (ERR_SUCCESS == lResult)
	{
		dwHi = (dwModuleID & 0xFFFF0000) >> 16;
		dwLow = dwModuleID & 0x0000FFFF;

		if (0 == dwLow)
			printf("/*** APAX %x module Sample code***/\n\n", (WORD)dwHi);
		else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
			printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
		else
			printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
	}
	else
		printf("Get module ID error, error code = %d\n", (WORD)lResult);

	usleep(100000);
		
	//Set inner timeout before config module
	printf("*** Set inner timeout before config module ***\n");
	wTimeout = 1000; //ms
	lResult = SYS_SetInnerTimeout(g_lHandle, wTimeout);
	if (ERR_SUCCESS == lResult)
		printf("Set inner timeout succeed, timeout = %d ms\n\n", wTimeout);
	else
		printf("Get inner timeout error, error code = %d\n\n", (WORD)lResult);
	
	wNewRangeTab[0] = NEG_10_TO_10_V;
	wNewRangeTab[1] = NEG_10_TO_10_V;
	wNewRangeTab[2] = NEG_10_TO_10_V;
	wNewRangeTab[3] = NEG_10_TO_10_V;
	wNewRangeTab[4] = NEG_10_TO_10_V;
	wNewRangeTab[5] = NEG_10_TO_10_V;
	wNewRangeTab[6] = NEG_10_TO_10_V;
	wNewRangeTab[7] = NEG_10_TO_10_V;

	printf("\n*** Get module ID and current range setting ***\n");
	//clear all channels range table first
	lResult = AIO_SetRanges(g_lHandle, g_usSlotNum, 8, wNewRangeTab);
	if (ERR_SUCCESS == lResult)
		printf("Set range succeed\n");
	else
		printf("Set range error, error code = %d\n", (WORD)lResult);

	//get module ID and current range setting
	lResult = SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo);
	if (ERR_SUCCESS == lResult)
	{
		My_Show_SlotInfo(&MySlotInfo);
	}
	else
		printf("Get slot information error, error code = %d\n", (WORD)lResult);

#ifdef ENABLE_CALIBRATION_PROCESS

	printf("\n***AO module calibration ***\n");
	//AO module calibration has 4 steps to do, and only one channel can be calibrated at once.
	//Step 1: Must set channel Range table among -10~10V, 5~5V, 0~20mA by calling AO_SetCalibrationMode() function
	//Step 2: Call AO_SetCalibrationMode() function
	//Step 3: AO module real output 
	//Step 4: Call AIO_SetZeroCalibration() or AIO_SetSpanCalibration() to set Zero or Span point
	
	//Step 1: Must set channel Range table among -10~10V, 5~5V, 0~20mA by calling AO_SetCalibrationMode() function
	wNewRangeTab[0] = NEG_10_TO_10_V; //take channel 0 as example

	lResult = AIO_SetRanges(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal,  wNewRangeTab);
	if (ERR_SUCCESS == lResult)
		printf("Set new range table succeed.\n");
	else
		printf("Set new range table error, error code = %d\n", (WORD)lResult);
	
	//Step 2: Call AO_SetCalibrationMode() function
	lResult = AO_SetCalibrationMode(g_lHandle, g_usSlotNum);
	if (ERR_SUCCESS == lResult)
		printf("Set calibration mode succeed.\n");
	else
		printf("Set calibration mode error, error code = %d\n", (WORD)lResult);

	//Step 3: AO module real output (Zero calibration)
	wChannel = 0;
	wValues[0] = 0x0; //set min
	lResult = AO_SetValue(g_lHandle, g_usSlotNum, wChannel, wValues[0]);
	if (ERR_SUCCESS == lResult)
			; //use voltmeter to measure if output value equal to -10V or not
	else
		printf("wChannel %d set value error, error code = %d\n", wChannel, (WORD)lResult);
	
	//Step 4: Call AIO_SetZeroCalibration() to set Zero point
	lResult = AIO_SetZeroCalibration(g_lHandle, g_usSlotNum, wChannel, 0);
	if (ERR_SUCCESS == lResult)
		printf("Set channel %d Zero calibration succeed.\n", wChannel);
	else
		printf("Set channel %d Zero calibration error, erro code = %d\n", wChannel, (WORD)lResult);

	usleep(200000);
	
	//Step 3: AO module real output (Span calibration) 
	wValues[0] = 0xFFFF; //Set max
	lResult = AO_SetValue(g_lHandle, g_usSlotNum, wChannel, wValues[0]);
	if (ERR_SUCCESS == lResult)
	{
		; //use voltmeter to measure if output value equal to 10V or not
	}
	else
		printf("wChannel %d set value error, erro code = %d\n", wChannel, (WORD)lResult);
	
	//Step 4: Call AIO_SetSpanCalibration() to set Span point
	lResult = AIO_SetSpanCalibration(g_lHandle, g_usSlotNum, wChannel, 0);
	if (ERR_SUCCESS == lResult)
		printf("Set channel %d Span calibration succeed.\n\n", wChannel);
	else
		printf("Set channel %d Span calibration error, erro code = %d\n\n", wChannel, (WORD)lResult);
#endif

	
	//Get channel status
	printf("*** Get channel status ***\n");
	lResult = AIO_GetChannelStatus(g_lHandle, g_usSlotNum, byChannelStatus);
	if (ERR_SUCCESS == lResult)
	{
		for (i=0; i<8; i++)
		{
			printf("Slot %d channel %d status is ", g_usSlotNum, i);

			switch(byChannelStatus[i])
			{
				case 0:
					printf("\"None\"\n");
					break;
				case 1:
					printf("\"Normal\"\n");
					break;
				case 2:
					printf("\"Over current\"\n");
					break;
				case 3:
					printf("\"Under current\"\n");
					break;
				case 4:
					printf("\"Burn out\"\n");
					break;
				case 5:
					printf("\"Open loop\"\n");
					break;
				case 6:
					printf("\"Not ready\"\n");
					break;
				default:
					printf("\"Unknown\"\n");
					break;
			}
		}
	}
	else
		printf("Get Slot %d channel status error, error code = %d\n", g_usSlotNum, (WORD)lResult);

	
	printf("\n*** Set new range table ***\n");
	wNewRangeTab[0] = NEG_10_TO_10_V;
	wNewRangeTab[1] = ZERO_TO_5_V;
	wNewRangeTab[2] = NEG_2P5_TO_2P5_V;
	wNewRangeTab[3] = NEG_2P5_TO_2P5_V;
	wNewRangeTab[4] = _4_TO_20_MA;
	wNewRangeTab[5] = NEG_5_TO_5_V;
	wNewRangeTab[6] = ZERO_TO_20_MA;
	wNewRangeTab[7] = NEG_2P5_TO_2P5_V;

	//Set range follow APAX 5028 range table setting
	lResult = AIO_SetRanges(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal,  wNewRangeTab);
	if (ERR_SUCCESS == lResult)
	{
		lResult = SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo);
		if (ERR_SUCCESS == lResult)
		{
			My_Show_SlotInfo(&MySlotInfo);
		}
		else
			printf("Get slot information error, erro code = %d\n", (WORD)lResult);
	}
	else
		printf("Set new range table error, error code = %d\n", (WORD)lResult);

	//Set inner timeout back to default
	printf("\n*** Set inner timeout back to default ***\n");
	wTimeout = 500; ///!!!+++ Yaojen@ Dec 2016:50; //ms
	lResult = SYS_SetInnerTimeout(g_lHandle, wTimeout);
	if (ERR_SUCCESS == lResult )
		printf("Set integration time succeed, timeout = %d ms.\n", wTimeout);
	else
		printf("Set integration time error, error code = %d.\n", (WORD)lResult);
		
	usleep(100000);
	
	printf("\n*** Get startup values ***\n");
	//clear all channels startup values first
	lResult = AO_SetStartupValues(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal, wStartupVal);
	if (ERR_SUCCESS == lResult)
		printf("Set Startup Values succeed.\n");
	else
		printf("Set Startup Values error, erro code = %d\n\n", (WORD)lResult);
	
	//Get startup values
	lResult = AO_GetStartupValues(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal, wStartupVal);
	if (ERR_SUCCESS == lResult)
	{
		for (wChannel=0; wChannel<MySlotInfo.byHwIoTotal_0; wChannel++)
		{
			Scale_RawData(&MySlotInfo, wChannel, wStartupVal);
		}
	}
	else
		printf("Get startup values error, error code = %d\n", (WORD)lResult);
	
	usleep(100000);

	printf("\n*** Set startup values ***\n");
	//Set startup values
	wStartupVal[0] = 0x1000;
	wStartupVal[1] = 0x2000;
	wStartupVal[2] = 0x3000;
	wStartupVal[3] = 0x4000;
	wStartupVal[4] = 0x5000;
	wStartupVal[5] = 0x6000;
	wStartupVal[6] = 0x7000;
	wStartupVal[7] = 0x9000;

	lResult = AO_SetStartupValues(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal, wStartupVal);
	if (ERR_SUCCESS == lResult)
	{
		//Get startup values
		lResult = AO_GetStartupValues(g_lHandle, g_usSlotNum, MySlotInfo.byChTotal, wStartupVal);
		if (ERR_SUCCESS == lResult)
		{
			for (wChannel=0; wChannel<MySlotInfo.byHwIoTotal_0; wChannel++)
			{
				Scale_RawData(&MySlotInfo, wChannel, wStartupVal);
			}
		}
		else
			printf("Get startup values error, error code =%d\n", (WORD)lResult);
	}
	else
		printf("Set startup values error, error code =%d\n", (WORD)lResult);

	//set value (single channel)
	printf("\n*** Set value (single channel)***\n");
	wValues[0] = 0x0800;
	wChannel = 0;

	lResult = AO_SetValue(g_lHandle, g_usSlotNum, wChannel, wValues[0]);
	if (ERR_SUCCESS != lResult)
		printf("Channel %d set value error, error code = %d\n", wChannel, (WORD)lResult);
	else
	{
		usleep(200000);

		wValues[0] = 0x0;
		lResult = AIO_GetValue(g_lHandle, g_usSlotNum, wChannel, wValues);
		if (ERR_SUCCESS != lResult)
			printf("Channel %d get value error, error code = %d\n", wChannel, (WORD)lResult);
		else
			Scale_RawData(&MySlotInfo, wChannel, wValues);
	}

	usleep(200000);

	//Set values (multi channels)
	printf("\n*** Set values (multi channels)***\n");
	printf("\nSet all channels output = 0x0500\n");
	wOutput[0] = 0x0500;
	wOutput[1] = 0x0500;
	wOutput[2] = 0x0500;
	wOutput[3] = 0x0500;
	wOutput[4] = 0x0500;
	wOutput[5] = 0x0500;
	wOutput[6] = 0x0500;
	wOutput[7] = 0x0500;

	lResult = AO_SetValues(g_lHandle, g_usSlotNum, dwChannelMask, wOutput);
	if (ERR_SUCCESS == lResult)
	{
		usleep(100000);
		
		lResult = AIO_GetValues(g_lHandle, g_usSlotNum, wValues);
		if (ERR_SUCCESS == lResult)
		{
			for (wChannel=0; wChannel<MySlotInfo.byHwIoTotal_0; wChannel++)
			{
				Scale_RawData(&MySlotInfo, wChannel, wValues);
			}
		}
		else
			printf("Get startup values error, error code =%d\n", (WORD)lResult);
	}
	else
		printf("Set startup values error, error code = %d\n", (WORD)lResult);

	//Buffer and flush values
	wOutput[0] = 0x2500;
	wOutput[1] = 0x2500;
	wOutput[2] = 0x2500;
	wOutput[3] = 0x2500;
	wOutput[4] = 0x2500;
	wOutput[5] = 0x2500;
	wOutput[6] = 0x2500;
	wOutput[7] = 0x2500;
	printf("\n*** Buffer and flush values ***\n");
	printf("\n*** Set all channels output = 0x2500 ***\n");

	lResult = AO_BufValues(g_lHandle, g_usSlotNum, dwChannelMask, wOutput);
	if (ERR_SUCCESS == lResult)
	{
		usleep(100000);

		lResult = OUT_FlushBufValues(g_lHandle, dwSlotMask);
		if (ERR_SUCCESS == lResult)
		{
			usleep(20000);
			AIO_GetValues(g_lHandle, g_usSlotNum, wValues);
			for (wChannel=0; wChannel<MySlotInfo.byHwIoTotal_0; wChannel++)
			{
				Scale_RawData(&MySlotInfo, wChannel, wValues);
			}
		}
		else
			printf("Flush values error, error code = %d\n", (WORD)lResult);
	}
	else
		printf("Buffer values error, error code =%d\n", (WORD)lResult);
	
	
	CloseLib();
	printf("/***        END         ***/ \n\n");
	
	
	return 0;
}
