// APAX_5040_24CH_DI.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
//#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"

//////////////////////////////////////////////
//Take APAX-5040 as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 


LONG g_lHandle = 0;
USHORT g_usSlotNum = 3;	//slot id from 0 to 31
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

int main(int argc, char* argv[])
{
	BOOL 	bValue = FALSE;
	WORD 	wChannel = 0;
	DWORD	dwHiValue = 0x0;
	DWORD	dwLowValue = 0x0;
	DWORD	dwModuleID = 0;
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	LONG	lResult = 0;
	DWORD   dwHighMask = 0;
	DWORD   dwLowMask = 0;
	DWORD	dwWidth = 0; //unit: 0.1 ms

	if (ADV_SUCCESS == OpenLib() )
	{
		printf("Slot ID setting in this sample code is %d.\n", g_usSlotNum);
		
		//get module ID
		lResult = SYS_GetModuleID(g_lHandle, g_usSlotNum, &dwModuleID);
		if (ERR_SUCCESS == lResult)
		{
			dwHi = (dwModuleID & 0xFFFF0000) >> 16;
			dwLow = dwModuleID & 0x0000FFFF;

			if (0 == dwLow)
				printf("/*** APAX %x module Sample code***/\n\n", (WORD)dwHi);
			else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
				printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
			else
				printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
		}
		else
			printf("Get module ID error, error code = %d\n", (WORD)lResult);

		usleep(100000);

		//Get DI filters
		printf("\n*** Get DI filters ***\n");
		lResult = DI_GetFilters(g_lHandle, g_usSlotNum, &dwHighMask, &dwLowMask, &dwWidth);
		if (ERR_SUCCESS == lResult)
			printf("Slot %d filter width = %u (0.1 ms)\n", g_usSlotNum, dwWidth);
		else
			printf("Get filters error, error code = %d\n", (WORD)lResult);

		usleep(100000);

		//Set DI filters
		printf("\n*** Set DI filters ***\n");
		dwHighMask = 0xFFFF; //Mask is always 0xFFFF
		dwLowMask = 0xFFFF; //Mask is always 0xFFFF
		dwWidth = 300; //0.1 ms

		lResult = DI_SetFilters(g_lHandle, g_usSlotNum, dwHighMask, dwLowMask, dwWidth);
		if (ERR_SUCCESS == lResult)	
			printf("Set slot %d filter width succeed.\n", g_usSlotNum);
		else
			printf("Fail to set filter width, error code = %d\n", (WORD)lResult);

		usleep(100000);

		//Get DI filters
		printf("\n*** Get DI filters again***\n");
		lResult = DI_GetFilters(g_lHandle, g_usSlotNum, &dwHighMask, &dwLowMask, &dwWidth);
		if (ERR_SUCCESS == lResult)
			printf("Slot %d filter width = %u ms\n", g_usSlotNum, dwWidth); 
		else
			printf("Fail to get DI filter, error code = %d\n", (WORD)lResult);

		usleep(100000);

		//Get single DI channel example
		printf("\n*** Get single DI channel ***\n");
		wChannel = 6; //from 0
		lResult = DIO_GetValue(g_lHandle, g_usSlotNum, wChannel, &bValue);
		if (ERR_SUCCESS == lResult)
		{
			if (bValue)
				printf("Channel %d = True\n", wChannel); //LED light on
			else
				printf("Channel %d = False\n", wChannel); //LED light off
		}
		else
			printf("Fail to get single DI channels, error code =%d\n", (WORD)lResult);

		usleep(100000);

		//Get multi DI channels example(single slot)
		printf("\n*** Get multi DI channels ***\n");
		lResult = DIO_GetValues(g_lHandle, g_usSlotNum, &dwHiValue, &dwLowValue);
		if (ERR_SUCCESS == lResult)
		{
			//show channel 0 ~ 23
			for (wChannel=0; wChannel<24; wChannel++)
			{
				if (dwLowValue & (0x0001 << wChannel) )
					printf("Channel %d = True\n", wChannel); //LED light on
				else
					printf("Channel %d = False\n", wChannel); //LED light off
			}
		}
		else
			printf("Fail to get multi DI channels, error code = %d\n", (WORD)lResult);
	}
	else
	{
		printf("Open library fail.\n");
	}
	
	CloseLib();
	printf("/***        END         ***/ \n\n");
	

	return 0;
}
