// APAX_5045_24CH_DIO.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
///#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"

//////////////////////////////////////////////
//Take APAX-5045 as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 


LONG g_lHandle = (LONG)0;
USHORT g_usSlotNum = 3;			//slot id from 0 to 31
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

int main(int argc, char* argv[])
{
	WORD	wChannel = 0;
	DWORD	dwHighValue = 0x0;
	DWORD	dwLowValue = 0x0;
	DWORD	dwModuleID = 0;
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	

	if (ADV_SUCCESS == OpenLib() )
	{
		printf("Slot ID setting in this sample code is %d.\n", g_usSlotNum);
		
		//get module ID
		SYS_GetModuleID(g_lHandle, g_usSlotNum, &dwModuleID);
		dwHi = (dwModuleID & 0xFFFF0000) >> 16;
		dwLow = dwModuleID & 0x0000FFFF;

		if (0 == dwLow)
			printf("/*** APAX %x module Sample code***/\n\n", (WORD)dwHi);
		else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
			printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
		else
			printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
	

		//Get multi DI channels example
		printf("\n*** Get multi DI channels  ***\n");
		
		if (ERR_SUCCESS == DIO_GetValues(g_lHandle, g_usSlotNum, &dwHighValue, &dwLowValue) )
		{
			//show channel 0 ~ 11
			for (wChannel=0; wChannel<12; wChannel++)
			{
				if (dwLowValue & (0x0001 << wChannel) )
					printf("Channel %d = True\n", wChannel); //LED light on
				else
					printf("Channel %d = False\n", wChannel); //LED light off
			}
		}
		else
			printf("Fail to get multi DI channels.\n");
		
		usleep(1000000);

		printf("\n*** Set single DO channel ***\n");
		//set single DO channel 
		wChannel = 7; //APAX 5045 DO channel from 0~11
		
		//Set DO channel TRUE or FALSE
		dwHighValue = 0x0;
		dwLowValue = 0x00000080; //enable channel 7
		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotNum, dwHighValue, dwLowValue) )
			printf("Channel %d = True\n", wChannel);	
		else
			printf("Fail to set DO value, channel %d.\n", wChannel);
		
		usleep(2000000);
		
		dwLowValue &= ~dwLowValue & 0x0000FFFF; //disable channel 7
		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotNum, dwHighValue, dwLowValue) )
			printf("Channel %d = False\n", wChannel);
		else
			printf("Fail to clear DO value, channel %d.\n", wChannel);
			
		usleep(2000000);

		//Set multi DO channels example
		printf("\n*** Set multi DO channels ***\n");
		
		dwHighValue = 0x0;
		dwLowValue = 0x00000F0F;
		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotNum, dwHighValue, dwLowValue) )
		{
			printf("Set multi DO channels successfully.\n");
			usleep(3000000);
		}
		else
			printf("Fail to set multi DO channels.\n");

		usleep(2000000);
		
		dwLowValue = 0x0; 
		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotNum, dwHighValue, dwLowValue) )
		{
			printf("Clear multi DO channels successfully.\n");
			usleep(3000000);
		}
		else
			printf("Fail to clear multi DO channels.\n");
			
	  ///!!!+++ Yaojen+ Dec 2016
	  dwHighValue = 0x0;
		dwLowValue = 0x00000555;		
		if (ERR_SUCCESS == /*DO_BufValues, DO_SetValues*/DO_BufValues(g_lHandle, g_usSlotNum, dwHighValue, dwLowValue) )
		{
			printf("Buffer multi DO channels successfully.\n");
			usleep(1000000);
		}
		else
			printf("Fail to buffer multi DO channels.\n");
			
		
		if( ERR_SUCCESS == OUT_FlushBufValues(g_lHandle, (1<<g_usSlotNum)) )
		{
			printf("Flush out multi DO channels successfully.\n");
		}
		else
			printf("Fail to flush out multi DO channels!\n");
			

	}
	else
	{
		printf("Open library fail.\n");
	}
	
	CloseLib();
	printf("/***        END         ***/ \n\n");
	

	return 0;
}
