// APAX_5046_24CH_DO.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
//#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"

//////////////////////////////////////////////
//Take APAX-5046 as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 


LONG g_lHandle = (LONG)NULL;
USHORT g_usSlotOneNum = 6;	//slot1: id from 0 to 31
USHORT g_usSlotTwoNum = 8;	//slot2(optional): id from 0 to 31
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

int main(int argc, char* argv[])
{
	BOOL bValue = TRUE;
	WORD	wChannel = 0;
	DWORD	dwHighValue = 0x0;
	DWORD	dwLowValue = 0x0;
	DWORD	dwSlotMask = ( (0x01 << g_usSlotOneNum) | (0x01 << g_usSlotTwoNum) );	//enable 2 slots
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	DWORD	dwModuleID = 0;


	if (ADV_SUCCESS == OpenLib() )
	{
		printf("Module One: Slot ID setting in this sample code is %d.\n", g_usSlotOneNum);
		printf("Module Two: Slot ID setting in this sample code is %d.\n", g_usSlotTwoNum);

		//get module ID
		SYS_GetModuleID(g_lHandle, g_usSlotOneNum, &dwModuleID);
		dwHi = (dwModuleID & 0xFFFF0000) >> 16;
		dwLow = dwModuleID & 0x0000FFFF;

		if (0 == dwLow)
			printf("/*** APAX %x module Sample code***/\n\n", (WORD)dwHi);
		else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
			printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
		else
			printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
		
		//Set single DO channel example
		printf("*** Set single DO channel ***\n");
		bValue = TRUE;
		wChannel = 10; //from 0

		if (ERR_SUCCESS == DO_SetValue(g_lHandle, g_usSlotOneNum, wChannel, bValue) )
			printf("Channel %d = True\n", wChannel);	
		else
			printf("Fail to set DO value, channel %d.\n", wChannel);
		
		usleep(2000000);
		
		bValue = FALSE;

		if (ERR_SUCCESS == DO_SetValue(g_lHandle, g_usSlotOneNum, wChannel, bValue) )
			printf("Channel %d = False\n", wChannel);	
		else
			printf("Fail to set DO value, channel %d.\n", wChannel);
			
		usleep(2000000);

		//Set multi DO channels example(single slot)
		printf("\n*** Set multi DO channels (single slot) ***\n");
		dwLowValue = 0x000F0F0F;
		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotOneNum, dwHighValue, dwLowValue) )
		{
			printf("Set multi DO channels successfully.\n");
			usleep(3000000);
		}
		else
			printf("Fail to set multi DO channels.\n");

		//Clean all DO channels(single slot)
		DO_SetValues(g_lHandle, g_usSlotOneNum, 0x0, 0x0);
		printf("Clear multi DO channels successfully.\n");

		usleep(2000000);

		//Set multi DO channels example(multi slots)
		printf("\n*** Set multi DO channels (multi slots) ***\n");
		//Buffer slot one (Set)
		if (ERR_SUCCESS == DO_BufValues(g_lHandle, g_usSlotOneNum, dwHighValue, dwLowValue) )
			printf("Slot: %d Buffer values successfully.\n", g_usSlotOneNum);
		else
			printf("Slot: %d Fail to buffer values.\n", g_usSlotOneNum);

		dwLowValue = 0x00F0F0F0;
		//Buffer slot two (Set)
		if (ERR_SUCCESS == DO_BufValues(g_lHandle, g_usSlotTwoNum, dwHighValue, dwLowValue) )
			printf("Slot: %d Buffer values successfully.\n", g_usSlotTwoNum);
		else
			printf("Slot: %d Fail to buffer values.\n", g_usSlotTwoNum);

		//Flush the buffered values
		if (ERR_SUCCESS == OUT_FlushBufValues(g_lHandle, dwSlotMask) )
		{
			printf("Flush buffered values successfully.\n");
			usleep(3000000);
		}
		else
			printf("Fail to flush buffered values.\n");


		//Buffer slot one (Clear)
		DO_BufValues(g_lHandle, g_usSlotOneNum, 0x0, 0x0);
		//Buffer slot two (Clear)
		DO_BufValues(g_lHandle, g_usSlotTwoNum, 0x0, 0x0);
		//Flush the buffered values
		OUT_FlushBufValues(g_lHandle, dwSlotMask);
		printf("Clear all buffered values.\n");
	}
	else
	{
		printf("Open library fail.\n");
	}
	
	CloseLib();
	printf("/***        END         ***/ \n\n");

	return 0;
}
