// APAX_5060_12CH_Relay.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
//#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"

//////////////////////////////////////////////
//Take APAX-5060 as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 


LONG g_lHandle = 0;
USHORT g_usSlotNum = 4;			//from 0 
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

int main(int argc, char* argv[])
{
	WORD 	wChannel  = 0;
	DWORD	dwHiValue = 0x0;
	DWORD	dwLowValue = 0x0;
	DWORD	SlotMask = (0x01 << g_usSlotNum);	//enable Slot 3
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	DWORD	dwModuleID = 0;


	if (ADV_SUCCESS == OpenLib() )
	{
		printf(" Slot ID setting in this sample code is %d.\n", g_usSlotNum);

		//get module ID
		SYS_GetModuleID(g_lHandle, g_usSlotNum, &dwModuleID);
		dwHi = (dwModuleID & 0xFFFF0000) >> 16;
		dwLow = dwModuleID & 0x0000FFFF;

		if (0 == dwLow)
			printf("/*** APAX %x module Sample code***/\n\n", (WORD)dwHi);
		else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
			printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
		else
			printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
		
		//Set single DO channel example
		printf("*** Set single DO channel ***\n");
		dwLowValue |= 0x0020; //enable channel 5
		for (wChannel =0; wChannel <32; wChannel ++)
		{
			if (dwLowValue & (0x00000001 << wChannel ) )
				break;
		}

		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotNum, dwHiValue, dwLowValue) )
			printf("Channel %d = True\n", wChannel );	
		else
			printf("Fail to set DO value, channel %d.\n", wChannel );
		
		usleep(2000000);
	
		dwLowValue &= 0xFFFFFFDF; //disable channel 5
		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotNum, dwHiValue, dwLowValue) )
			printf("Channel %d = False\n", wChannel );
		else
			printf("Fail to clear DO value, channel %d.\n", wChannel );
			
		usleep(2000000);

		//Set multi DO channels example
		printf("\n*** Set multi DO channels ***\n");
		printf("Use DO_SetValues() function.\n");
		
		dwLowValue = 0x000F0F;
		if (ERR_SUCCESS == DO_SetValues(g_lHandle, g_usSlotNum, dwHiValue, dwLowValue) )
		{
			printf("Set multi DO channels successfully.\n");
			usleep(3000000);
		}
		else
			printf("Fail to set multi DO channels.\n");

		//Clean all DO channels(single slot)
		DO_SetValues(g_lHandle, g_usSlotNum, 0x0, 0x0);
		printf("Clear multi DO channels successfully.\n");

		usleep(2000000);

		//Set multi DO channels example
		printf("\n*** Set multi DO channels ***\n");
		printf("Use DO_BufValues() function.\n");
		//Buffer slot one (Set)
		dwLowValue = 0x000000F0;
		if (ERR_SUCCESS == DO_BufValues(g_lHandle, g_usSlotNum, dwHiValue, dwLowValue) )
			printf("Buffer values successfully.\n");
		else
			printf("Fail to buffer values.\n");

		OUT_FlushBufValues(g_lHandle, SlotMask);

		usleep(2000000);

		//Buffer slot one (Clear)
		DO_BufValues(g_lHandle, g_usSlotNum, 0x0, 0x0);
		//Flush the buffered values
		OUT_FlushBufValues(g_lHandle, SlotMask);
		printf("Clear all buffered values.\n");

		
	}
	else
	{
		printf("Open library fail.\n");
	}
	
	CloseLib();
	printf("/***        END         ***/ \n\n");
	

	return 0;
}
