// APAX_5080_8CH_Counter.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
//#include "../include/adsdio.h"
#include "../include/apaxLocalIo/adsdio.h"

//////////////////////////////////////////////
//Take APAX-5080 as example 
//////////////////////////////////////////////
#define ADV_SUCCESS 0
#define ADV_FAIL	1 

#define NONE 0x00
#define DI	0x01
#define DO	0x02
#define AI	0x03
#define AO	0x04
#define COUNTER 0X05

//APAX 5080	range table
#define PULSE_DIR	0x01C0
#define UP_AND_DOWN	0x01C1
#define UP			0x01C2
#define FREQUENCY	0x01C3
#define AB_1X		0x01C4
#define AB_2X		0x01C5
#define AB_4X		0x01C6

LONG g_lHandle = 0;
USHORT g_usSlotNum = 3;			//slot id from 0 to 31
//////////////////////////////////////////////////////////////////////////
int OpenLib(void)
{
	DWORD dwVersion = 0;
	
	//Initialize the driver
	if(ERR_SUCCESS != ADAMDrvOpen(&g_lHandle) )
		return ADV_FAIL;		

	if (ERR_SUCCESS == SYS_GetVersion(g_lHandle, &dwVersion) )
		printf("/*** ADSDIO.lib version is %X\n\n", dwVersion);
	else
		return ADV_FAIL;

	return ADV_SUCCESS;
}

void CloseLib(void)
{
	//Terminate the driver
	if(0 != g_lHandle)
	{
		ADAMDrvClose(&g_lHandle);
		g_lHandle = 0;
	}
}

void GetChannelMode(struct SlotInfo *i_pSlotInfo, BYTE byCounterChNum)
{
	int iChIdx = 0;
	WORD wRangeType = 0;

	for (iChIdx=0; iChIdx<byCounterChNum; iChIdx++)
	{
		wRangeType = *( (i_pSlotInfo->wChRange) + iChIdx);
		switch(wRangeType)
		{
			case PULSE_DIR:
				printf("Channel %d is pulse mode.\n", iChIdx);
				break;

			case UP_AND_DOWN:
				printf("Channel %d is Up and Down mode.\n", iChIdx);
				break;

			case UP:
				printf("Channel %d is Up mode.\n", iChIdx);
				break;

			case FREQUENCY:
				printf("Channel %d is Frequency mode.\n", iChIdx);
				break;

			case AB_1X:
				printf("Channel %d is AB1X mode.\n", iChIdx);
				break;

			case AB_2X:
				printf("Channel %d is AB2X mode.\n", iChIdx);
				break;

			case AB_4X:
				printf("Channel %d is AB4X mode.\n", iChIdx);
				break;

			default:
				printf("Channel %d is unknow mode.\n", iChIdx);
				break;
		}
	}
}

void My_Show_SlotInfo(struct SlotInfo *i_pSlotInfo, BYTE *o_pCounterChNum)
{
	DWORD	dwHi = 0x0;
	DWORD	dwLow = 0x0;
	BYTE 	byTotalChOfSlot = 0;
	BYTE 	*pChNumber = 0;
	BYTE 	byChNumber = 0;
	int 	iAddrShift = 0;
	BYTE 	byChannelType = 0;

	dwHi = (i_pSlotInfo->dwModName & 0xFFFF0000) >> 16;
	dwLow = i_pSlotInfo->dwModName & 0x0000FFFF;

	//show module name
	if (0 == dwLow)
		printf("APAX %x module\n", (WORD)dwHi);
	else if (0 != (dwLow & 0x0000FF00) && (dwLow & 0x000000FF) )
		printf("/*** APAX %x%c%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8), (BYTE)(dwLow & 0x000000FF) );
	else
		printf("/*** APAX %x%c module Sample code***/\n\n", (WORD)dwHi, (BYTE)(dwLow >> 8) );
	
	//get total channel number of this slot
	byTotalChOfSlot = i_pSlotInfo->byChTotal;

	for (iAddrShift=0; iAddrShift<10; iAddrShift+=2)
	{
		pChNumber = (BYTE *)&(i_pSlotInfo->byHwIoType_0) + iAddrShift + 1;
		byChNumber += *pChNumber;
		
		byChannelType = *(pChNumber - 1);
		switch(byChannelType) 
		{
			case DI:
				//When iAddrShift = 0, it try to extract byHwIoTotal_0 value of struct SlotInfo
				printf("This module has %d DI channels.\n", *pChNumber);
				break;
			
			case DO:
				printf("This module has %d DO channels.\n", *pChNumber);
				break;
			
			case AI:
				printf("This module has %d AI channels.\n", *pChNumber);
				break;

			case AO:
				printf("This module has %d AO channels.\n", *pChNumber);
				break;

			case COUNTER:
				printf("This module has %d Counter channels.\n", *pChNumber);
				*o_pCounterChNum = *pChNumber;
				break;
		}

		if (byTotalChOfSlot <= byChNumber)
			break;
	}
	
	printf("\n");

	
	GetChannelMode(i_pSlotInfo, *o_pCounterChNum);
}
int main(int argc, char* argv[])
{
	DWORD dwValues[32] = {0};
	struct SlotInfo MySlotInfo ={0};
	WORD wNewRangeTab[8] = {0}; //APAX 5080 is a 8 channels counter module
	int i = 0;
	BYTE byCounterChNum = 0;

	printf("Slot ID setting in this sample code is %d.\n", g_usSlotNum);
	
	if (ADV_SUCCESS == OpenLib() )
	{
		printf("\n*** Get module ID and current range setting ***\n");
		//get module ID and current range setting
		if (ERR_SUCCESS == SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo) )
		{
			My_Show_SlotInfo(&MySlotInfo, &byCounterChNum);
		}
		else
			printf("Get slot information error.\n");
		
		usleep(100000);

		//Get every channel current (counter) value
		printf("\nGet every channel current (counter) value.\n");
		if (ERR_SUCCESS == CNT_GetValues(g_lHandle, g_usSlotNum, dwValues) )
		{
			for (i=0; i<byCounterChNum; i++)
			{
				printf("Channel %d counter value = %u\n", i, (WORD)dwValues[i]);	
			}
		}
		else
			printf("Call CNT_GetValues() function error.\n");
		
		usleep(100000);

		//Set new channel range mode
		printf("\nSet new channel range mode.\n");

		//important!!!
		// Up/Down mode must set in a pair of channels
		wNewRangeTab[0] = UP_AND_DOWN; 
		wNewRangeTab[1] = UP_AND_DOWN; 
		
		wNewRangeTab[2] = UP;	 			
		wNewRangeTab[3] = FREQUENCY;

		// AB phase mode must set in a pair of channels
		wNewRangeTab[4] = AB_1X;
		wNewRangeTab[5] = AB_1X;

		wNewRangeTab[6] = FREQUENCY;
		wNewRangeTab[7] = UP;
		CNT_SetRanges(g_lHandle, g_usSlotNum, byCounterChNum, wNewRangeTab);

		usleep(100000);

		//Get new channel range mode
		printf("\nShow new channel range mode.\n");
		if (ERR_SUCCESS == SYS_GetSlotInfo(g_lHandle, g_usSlotNum, &MySlotInfo) )
			GetChannelMode(&MySlotInfo, byCounterChNum);
		else
			printf("\nShow new channel range error.\n");

		usleep(100000);

		//Get every channel current (counter) value
		printf("\nNow get every channel current (counter) value again.\n");

		//Flash values after change range mode
		if (ERR_SUCCESS == CNT_ClearValues(g_lHandle, g_usSlotNum, 0xFFFFFFFF))
		{
			usleep(100000);
			if (ERR_SUCCESS == CNT_GetValues(g_lHandle, g_usSlotNum, dwValues) )
			{
				for (i=0; i<byCounterChNum; i++)
				{
					printf("Channel %d counter value = %u\n", i, (WORD)dwValues[i]);	
				}
			}
			else
				printf("Call CNT_GetValues() function error.\n");
		}
		

	}
	else
	{
		printf("Open library fail.\n");
	}
	
	CloseLib();
	printf("/***        END         ***/ \n\n");
	
	return 0;
}
